@@ -1,12 +1,17 @@
+from lightwood.encoders.encoder_base import BaseEncoder
 from lightwood.encoders.datetime.datetime import DatetimeEncoder
 from lightwood.encoders.image.img_2_vec import Img2VecEncoder
 from lightwood.encoders.image.nn import NnAutoEncoder
 from lightwood.encoders.numeric.numeric import NumericEncoder
 from lightwood.encoders.text.infersent import InferSentEncoder
+from lightwood.encoders.text.distilbert import DistilBertEncoder
 from lightwood.encoders.text.rnn import RnnEncoder
 from lightwood.encoders.categorical.onehot import OneHotEncoder
 from lightwood.encoders.categorical.autoencoder import CategoricalAutoEncoder
 from lightwood.encoders.time_series.ts_fresh_ts import TsFreshTsEncoder
+from lightwood.encoders.time_series.rnn import RnnEncoder as TsRnnEncoder
+# from lightwood.encoders.audio.amplitude_ts import AmplitudeTsEncoder
+from lightwood.encoders.categorical.multihot import MultiHotEncoder
 
 try:
     from lightwood.encoders.time_series.cesium_ts import CesiumTsEncoder
@@ -15,12 +20,6 @@ except:
     export_cesium = False
     print('Failed to export cesium timeseires encoder')
 
-try:
-    from lightwood.encoders.audio.audio import AmplitudeTsEncoder
-    export_ampl = True
-except:
-    export_ampl = False
-    print('Failed to export amplitude audio encoder')
 
 class DateTime:
     DatetimeEncoder = DatetimeEncoder
@@ -45,12 +44,10 @@ class Categorical:
 
 class TimeSeries:
     TsFreshTsEncoder = TsFreshTsEncoder
+    RnnEncoder = TsRnnEncoder
     if export_cesium:
         CesiumTsEncoder = CesiumTsEncoder
 
-class Audio:
-    if export_ampl:
-        AmplitudeTsEncoder = AmplitudeTsEncoder
 
 class BuiltinEncoders:
     DateTime = DateTime
@@ -59,7 +56,7 @@ class BuiltinEncoders:
     Text = Text
     Categorical = Categorical
     TimeSeries = TimeSeries
-    Audio = Audio
+    # Audio = Audio
 
 
 BUILTIN_ENCODERS = BuiltinEncoders
