@@ -1,13 +1,9 @@
 import datetime
 import torch
+from lightwood.encoders.encoder_base import BaseEncoder
 
 
-class DatetimeEncoder:
-
-    def __init__(self, is_target=False):
-        self._pytorch_wrapper = torch.FloatTensor
-        self._prepared = False
-
+class DatetimeEncoder(BaseEncoder):
     def prepare_encoder(self, priming_data):
         if self._prepared:
             raise Exception('You can only call "prepare_encoder" once for a given encoder.')
