@@ -4,16 +4,15 @@ import sys
 
 import torch
 import numpy as np
+from lightwood.encoders.encoder_base import BaseEncoder
 
 
-class NumericEncoder:
+class NumericEncoder(BaseEncoder):
 
     def __init__(self, data_type=None, is_target=False):
+        super().__init__(is_target)
         self._type = data_type
         self._mean = None
-        self._pytorch_wrapper = torch.FloatTensor
-        self._prepared = False
-        self.is_target = is_target
         self.decode_log = False
         self.extra_outputs = 0
 
