@@ -223,7 +223,7 @@ class NnMixer:
                             stop_training = True
 
                         # If the trauining subset is overfitting on it's associated testing subset
-                        if (subset_delta_mean <= 0 and len(subset_test_error_delta_buff) > 4) or (time.time() - started_subset) > stop_training_after_seconds/len(train_ds.subsets.keys()):
+                        if (subset_delta_mean <= 0 and len(subset_test_error_delta_buff) > 4) or (time.time() - started_subset) > stop_training_after_seconds/len(train_ds.subsets.keys()) or subset_test_error < 0.001:
                             logging.info('Finished fitting on {subset_id} of {no_subsets} subset'.format(subset_id=subset_id, no_subsets=len(train_ds.subsets.keys())))
 
                             if self.is_selfaware:
@@ -514,7 +514,12 @@ class NnMixer:
 
             input_sample, output_sample = ds[0]
 
-            self.net = self.nn_class(self.dynamic_parameters, input_size=len(input_sample), output_size=len(output_sample), nr_outputs=len(self.out_types), selfaware=False, deterministic=self.config['mixer']['deterministic'])
+            self.net = self.nn_class(self.dynamic_parameters,
+                                     input_size=len(input_sample),
+                                     output_size=len(output_sample),
+                                     nr_outputs=len(self.out_types),
+                                     selfaware=False,
+                                     deterministic=self.config['mixer']['deterministic'])
             self.net = self.net.train()
 
             if self.batch_size < self.net.available_devices:
@@ -529,11 +534,25 @@ class NnMixer:
                     output_weights = torch.Tensor(ds.output_weights).to(self.net.device)
                 else:
                     output_weights = None
-                for output_type in self.out_types:
-                    if output_type in (COLUMN_DATA_TYPES.CATEGORICAL):
-                        self.criterion_arr.append(TransformCrossEntropyLoss(weight=output_weights))
-                        self.unreduced_criterion_arr.append(TransformCrossEntropyLoss(weight=output_weights,reduce=False))
-                    elif output_type in (COLUMN_DATA_TYPES.NUMERIC):
+
+                for k, output_type in enumerate(self.out_types):
+                    if output_type == COLUMN_DATA_TYPES.CATEGORICAL:
+                        if output_weights is None:
+                            weights_slice = None
+                        else:
+                            weights_slice = output_weights[ds.out_indexes[k][0]:ds.out_indexes[k][1]]
+
+                        self.criterion_arr.append(TransformCrossEntropyLoss(weight=weights_slice))
+                        self.unreduced_criterion_arr.append(TransformCrossEntropyLoss(weight=weights_slice,reduce=False))
+                    elif output_type == COLUMN_DATA_TYPES.MULTIPLE_CATEGORICAL:
+                        if output_weights is None:
+                            weights_slice = None
+                        else:
+                            weights_slice = output_weights[ds.out_indexes[k][0]:ds.out_indexes[k][1]]
+
+                        self.criterion_arr.append(torch.nn.BCEWithLogitsLoss(weight=weights_slice))
+                        self.unreduced_criterion_arr.append(torch.nn.BCEWithLogitsLoss(weight=weights_slice, reduce=False))
+                    elif output_type == COLUMN_DATA_TYPES.NUMERIC:
                         self.criterion_arr.append(QuantileLoss(quantiles=self.quantiles))
                         self.unreduced_criterion_arr.append(QuantileLoss(quantiles=self.quantiles, reduce=False))
                     else:
@@ -611,6 +630,7 @@ class NnMixer:
                 loss = None
                 for k, criterion in enumerate(self.criterion_arr):
                     target_loss = criterion(outputs[:,ds.out_indexes[k][0]:ds.out_indexes[k][1]], labels[:,ds.out_indexes[k][0]:ds.out_indexes[k][1]])
+
                     if loss is None:
                         loss = target_loss
                     else:
